-- factory.lua (Entity)
-- @author Gerard Purra Teixido <gerard.purra@gmail.com>

local Messenger = require 'src/utils/messenger'
local Player = require 'src/entities/player'
local Block = require 'src/entities/block'
local Slug = require 'src/entities/enemies/slug'

local EntityFactory = {
  switch = {
    -- SLUG
    slug = function(data)
      local instance = Slug.new(data)

      local collider = CollisionManager.hc:rectangle(
        instance.x, instance.y, instance.w, instance.h
      )
      collider.entity = instance
      collider.mode = Globals.COLLIDER_DYNAMIC
      collider.group = Globals.MASK_ENEMY
      collider.mask = {
        [Globals.MASK_ENEMY] = true,
        [Globals.MASK_PLAYER] = true,
        [Globals.MASK_BLOCK] = true
      }
      collider.is_active = true
      CollisionManager:add(collider)

      local trigger = TriggerManager.hc:rectangle(
        instance.x, instance.y, instance.w, instance.h
      )
      trigger.entity = instance
      trigger.is_active = true
      trigger.group = Globals.MASK_ENEMY
      trigger.mask = {
        [Globals.MASK_PLAYER] = true
      }
      local messenger = Messenger.new()
      messenger.trigger = trigger
      trigger.messenger = messenger
      TriggerManager:add(trigger)
      
      local trigger_alert = TriggerManager.hc:circle(
        instance.x + instance.w / 2, instance.y + instance.h / 2,
        Globals.SLUG_ALERT
      )
      trigger_alert.is_active = true
      trigger_alert.group = Globals.MASK_NOTHING 
      trigger_alert.mask = {
        [Globals.MASK_PLAYER] = true
      }
      local messenger_alert = Messenger.new()
      messenger_alert.trigger = trigger_alert
      trigger_alert.messenger = messenger_alert
      TriggerManager:add(trigger_alert)

      instance.collider = collider
      instance.trigger = trigger
      instance.trigger_alert = trigger_alert
      return instance
    end
  }
}

function EntityFactory.new(entity, data)
  if EntityFactory.switch[entity] then
    return EntityFactory.switch[entity]( data )
  end
end

return EntityFactory
