-- slug.lua
-- @author Gerard Purra Teixido <gerard.purra@gmail.com>

local Slug = {
  -- position
  x = 0,
  y = 0,
  -- direction
  dx = 0,
  dy = 0,
  -- facing
  facing = 0,
  -- dimensions
  w = 0,
  h = 0,
  -- speed
  current_speed = 0,
  walk_speed = 80,
  run_speed = 200,
  -- state
  current_state = 0,
  -- target
  target = nil,
  -- path
  path = nil,
  path_length = 0,
  current_path_node = 0,
  -- collider
  collider = nil,
  pushing_top = false,
  pushing_bottom = false,
  pushing_left = false,
  pushing_right = false,
  -- triggers
  trigger = nil,
  trigger_alert = nil
}
local slug_mt = { __index = Slug }

function Slug.new(data)
  local instance = data
  return setmetatable(instance, slug_mt)
end

function Slug:update(dt)
  self:readInbox()
  self.trigger.messenger:clear()
  self.trigger_alert.messenger:clear()

  -- STANDBY
  if self.current_state == Globals.ENEMY_STATE_STANDBY then
    return
  end
  -- ALERT
  if self.current_state == Globals.ENEMY_STATE_ALERT then
    -- Get self and target tile position
    local tmx, tmy = Map:convertPixelToTile(self.target.x, self.target.y)
    local smx, smy = Map:convertPixelToTile(self.x, self.y)

    -- If self has path to target
    if self.path ~= nil then
      local lastNode = self.path[self.path_length]
      -- If target no longer on last node then set path to nil
      if node.x ~= tmx or node.y ~= tmy then
        self.path = nil
        self.current_path_node = 0
        self.path_length = 0
        return
      end

      local currentNode = self.path[self.current_path_node]
      if currentNode.x == smx and currentNode.y == smy then
        self.current_path_node = self.current_path_node + 1
      end

    end
    -- If path is nil calculate path
    if self.path == nil then
      self.path, self.path_length = Pathfinder:getPath(smx, smy, tmx, tmy)
      self.current_path_node = 1
    end
    
    local dx, dy = self:followPath()
    local velocity = self.walk_speed * dt
    self:move(dx * velocity, dy * velocity)
    return
  end
  -- ATTACK
  if self.current_state == Globals.ENEMY_STATE_ATTACK then
    
    return
  end
end

function Slug:draw()
end

function Slug:remove()
  CollisionManager:remove(self.collider)  
  TriggerManager:remove(self.trigger)
  TriggerManager:remove(self.trigger_alert)
  self.target = nil
  self = nil
end

function Slug:readInbox()
  -- STANDBY
  if self.current_state == Globals.ENEMY_STATE_STANDBY then
    if self.trigger_alert.messenger:has_inbox() then
      self.target = self.trigger_alert.messenger:read()
      self.current_state = Globals.ENEMY_STATE_ALERT 
    end
    return
  end
  -- ALERT
  if self.current_state == Globals.ENEMY_STATE_ALERT then
    if not self.trigger_alert.messenger:has_inbox() then
      self.target = nil
      self.current_state = Globals.ENEMY_STATE_STANDBY 
    end
    if self.trigger_attack.messenger:has_inbox() then
      self.current_state = Globals.ENEMY_STATE_ATTACK
    end
    return
  end
  -- ATTACK
  if self.current_state == Globals.ENEMY_STATE_ATTACK then
    if not self.trigger_attack.messenger:has_inbox() then
      self.curren_state = Globals.ENEMY_STATE_ALERT
    end
    return
  end
end

function Slug:move(dx, dy)
  self.x, self.y = self.x + dx, self.y + dy
  self.collider:move(dx, dy)
  self.trigger:move(dx, dy)
  self.trigger_alert:move(dx, dy)
end

function Slug:center()
  return self.collider:center()
end

function Slug:followPath()
  local current_node = self.path[self.current_path_node]
  local x1, y1 = Map:convertTileToPixel(current_node.x, current_node.y)
  local x2, y2 = self:center()
  local dx, dy = Vector.sub(x1, y1, x2, y2)
  return Vector.normalize(dx, dy)
end

function Slug:draw_debug()
  love.graphics.setColor(0, 255, 0, 255)
  love.graphics.print('CURRENT_STATE: ' .. self.current_state, 500, 50)
  love.graphics.print('FACING: ' .. self.facing, 500, 70)
  love.graphics.print('MESSAGES: ' .. self.trigger_alert.messenger.message_count, 500, 90)
end

return Slug
