-- factory.lua (Entity)
-- @author Gerard Purra Teixido <gerard.purra@gmail.com>

local Messenger = require 'src/utils/messenger'
local Player = require 'src/entities/player'
local Block = require 'src/entities/block'
local Slug = require 'src/entities/enemies/slug'

local EntityFactory = {
  switch = {
    -- PLAYER
    player = function(data)
      local instance = Player.new(data)

      local collider = CollisionManager.hc:rectangle(
        instance.x, instance.y, instance.w, instance.h
      )
      collider.entity = instance
      collider.mode = Globals.COLLIDER_DYNAMIC
      collider.group = Globals.MASK_PLAYER
      collider.mask = {
        [Globals.MASK_ENEMY] = true,
        [Globals.MASK_PLAYER] = true,
        [Globals.MASK_BLOCK] = true,
        [Globals.MASK_ITEM] = true
      }
      collider.is_active = true
      CollisionManager:add(collider)

      local trigger = TriggerManager.hc:rectangle(
        instance.x, instance.y, instance.w, instance.h
      )
      trigger.entity = instance
      trigger.is_active = true
      trigger.group = Globals.MASK_PLAYER
      trigger.mask = {
        [Globals.MASK_ENEMY] = true
      }
      local messenger = Messenger.new()
      trigger.messenger = messenger
      TriggerManager:add(trigger)

      instance.collider = collider
      instance.trigger = trigger

      return instance
    end,
    -- BLOCK
    block = function(data)
      local instance = Block.new(data)

      local collider = CollisionManager.hc:rectangle(
        instance.x, instance.y, instance.w, instance.h
      )
      collider.entity = instance
      collider.mode = Globals.COLLIDER_STATIC
      collider.group = Globals.MASK_BLOCK
      collider.mask = {
        [Globals.MASK_ENEMY] = true,
        [Globals.MASK_PLAYER] = true
      }
      collider.is_active = true
      CollisionManager:add(collider)

      instance.collider = collider
      return instance
    end
  }
}

function EntityFactory.new(entity, data)
  if EntityFactory.switch[entity] then
    return EntityFactory.switch[entity]( data )
  end
end

return EntityFactory
