-- event_manager.lua
-- @author Gerard Purra Teixido <gerard.purra@gmail.com>

local TriggerManager = {
  hc = nil,
  triggers = nil
}

function TriggerManager:init(hc)
  self.hc = hc
  self.triggers = {}
end

function TriggerManager:add(trigger)
  table.insert(self.triggers, trigger)
end

function TriggerManager:update()
  self:resolveTriggers()
end

-- TriggerManager:remove - Removes trigger instance
-- @param trigger - Trigger instance to be removed
function TriggerManager:remove(trigger)
  self.hc:remove(trigger)
  trigger.remove = true
  for i, v in ipairs(self.triggers) do
    if v.remove then
      table.remove(self.triggers, i)
      return
    end
  end
end

-- TriggerManager:clear - Clear all tables to reset triggers
function TriggerManager:clear()
  self.hc:resetHash(200)
  for i, v in ipairs(self.triggers) do
    self.triggers[i] = nil
  end
end

function TriggerManager:resolveTriggers()
  for _, trg in ipairs(self.triggers) do
    if trg.is_active then
      local collisions = self.hc:collisions(trg)
      for other, _ in pairs(collisions) do
        if other.is_active
          and (trg.mask[other.group] or trg.mask[Globals.MASK_ALL]) then
          trg.messenger:send(other.entity)
          -- other.entity.messenger:send(trg.entity)
        end
      end
    end
  end
end

function TriggerManager:debug_draw()
  love.graphics.setColor(255, 255, 0, 255)
  for i, collider in ipairs(self.triggers) do
    if collider.is_active then
      collider:draw('line')
    end
  end
  love.graphics.setColor(255, 255, 255, 255)
end

return TriggerManager
